/*
 * sockets.c	Supporting routines for communication via sockets
 *
 *              Created by Dr. Xin Yuan for project I in the
 *              Computer Networks class (cis49306).
 *              Modify on your own risk. 
 */

#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/time.h>
#include <netdb.h>
#include <errno.h>

#include "sockets.h"

int
open_bound_tcp_socket (int *port)
{
  int                 s;
  struct sockaddr_in  addr;
  int                 len = sizeof(addr);

  if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    {
      perror(": can't get socket");
      _exit(1);
    }

  addr.sin_addr.s_addr = INADDR_ANY;
  addr.sin_family      = AF_INET;
  addr.sin_port        = *port;

  if (bind(s, (struct sockaddr *) &addr, len) < 0)
    {
      perror(": bind");
      close(s);
      _exit(1);
    }

  if (getsockname(s, (struct sockaddr *)&addr, &len) < 0)
    {
      perror(": can't get name");
      close(s);
      _exit(1);
    }

  *port = addr.sin_port;
  return s;
}

int
tcp_connect (char *host, int port)
{
  struct sockaddr_in addr;
  int                sock;
  struct hostent *   hp;
  struct sockaddr_in cl_addr;


  if ((hp = gethostbyname(host)) == 0)
    {
      printf("sockets: can't get host name\n");
      exit(1);
    }
  
  bcopy(hp->h_addr, &cl_addr.sin_addr, hp->h_length);
  cl_addr.sin_family = AF_INET;
  cl_addr.sin_port = port;
  
  if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    {
      perror(": can't get socket");
      exit(1);
    }
  
  if (connect(sock, (struct sockaddr *)&cl_addr, sizeof(cl_addr)) < 0)
    {
      perror(": connect");
      exit(1);
    }
  
  return (sock);
}


void
init_tcp_accept (int acc_sock, fd_set *dset)
{
  listen(acc_sock, 5);

  FD_ZERO(dset);
  FD_SET(acc_sock, dset);
}


int 
tcp_write (int sock, char *msgp, unsigned msgsize)
{
  int rlen;

  do 
    {
      rlen = write(sock, (caddr_t) msgp, msgsize);

    } while (rlen == -1 && errno == EINTR);
      
  if (rlen == 0)
    {
      /* connection closed */
      
      return 0;
    }

  if (rlen < msgsize)
    {
      perror(": write");
      _exit(1);
    }

  return !0;
}  


int
tcp_read (int sock, char *msgp, unsigned msgsize)
{
  int rlen;
  int temp;

  do 
    {
      rlen = read(sock, (caddr_t) msgp, msgsize);
      
    } while (rlen == -1 && errno == EINTR);
      
  if (rlen == 0)
    {
      /* connection closed */
      
      return 0;
    }


   while (rlen <msgsize) {
        printf("begin rlen= %d\n", rlen);
       do {
          temp = read(sock, (caddr_t) (msgp+rlen), msgsize-rlen);
      
        } while (temp == -1 && errno == EINTR);
        rlen += temp;
        printf("end rlen= %d\n", rlen);
   }
      
  if (rlen == 0)
    {
      /* connection closed */
      
      return 0;
    }
  
   if (rlen < msgsize)
    {
printf("rlen = %d\n", rlen);
      perror(": read");
      _exit(1);
    } 
  
  return !0;
}



